@@ -1,4 +1,4 @@
-import pickle
+import logging
 import sys
 import warnings
 from contextlib import contextmanager
@@ -15,19 +15,33 @@ from typing import (
 )
 
 from .lock import lock
+from .serializers import PickleSerializer, SharedMemoryDictSerializer
 from .templates import MEMORY_NAME
 
 NOT_GIVEN = object()
+DEFAULT_SERIALIZER = PickleSerializer()
+
+
+logger = logging.getLogger(__name__)
 
 
 class SharedMemoryDict:
-    def __init__(self, name: str, size: int) -> None:
+    def __init__(
+        self,
+        name: str,
+        size: int,
+        *,
+        serializer: SharedMemoryDictSerializer = DEFAULT_SERIALIZER,
+    ) -> None:
         super().__init__()
+        self._serializer = serializer
         self._memory_block = self._get_or_create_memory_block(
             MEMORY_NAME.format(name=name), size
         )
 
     def cleanup(self) -> None:
+        if not hasattr(self, '_memory_block'):
+            return
         self._memory_block.close()
 
     def move_to_end(self, key: str, last: Optional[bool] = True) -> None:
@@ -116,13 +130,13 @@ class SharedMemoryDict:
     def get(self, key: str, default: Optional[Any] = None) -> Any:
         return self._read_memory().get(key, default)
 
-    def keys(self) -> KeysView[Any]:  # type: ignore
+    def keys(self) -> KeysView[Any]:
         return self._read_memory().keys()
 
-    def values(self) -> ValuesView[Any]:  # type: ignore
+    def values(self) -> ValuesView[Any]:
         return self._read_memory().values()
 
-    def items(self) -> ItemsView:  # type: ignore
+    def items(self) -> ItemsView:
         return self._read_memory().items()
 
     def pop(self, key: str, default: Optional[Any] = NOT_GIVEN):
@@ -145,17 +159,25 @@ class SharedMemoryDict:
         try:
             return SharedMemory(name=name)
         except FileNotFoundError:
-            return SharedMemory(name=name, create=True, size=size)
+            shm = SharedMemory(name=name, create=True, size=size)
+            data = self._serializer.dumps({})
+            shm.buf[: len(data)] = data
+            return shm
 
     def _save_memory(self, db: Dict[str, Any]) -> None:
-        data = pickle.dumps(db, pickle.HIGHEST_PROTOCOL)
+        data = self._serializer.dumps(db)
         try:
-            self._memory_block.buf[: len(data)] = data  # type: ignore
+            self._memory_block.buf[: len(data)] = data
         except ValueError as exc:
             raise ValueError("exceeds available storage") from exc
 
     def _read_memory(self) -> Dict[str, Any]:
         try:
-            return pickle.loads(self._memory_block.buf)
-        except pickle.UnpicklingError:
+            return self._serializer.loads(self._memory_block.buf.tobytes())
+        except Exception as exc:
+            logger.warning(f"Fail to load data: {exc!r}")
             return {}
+
+    @property
+    def shm(self) -> SharedMemory:
+        return self._memory_block
